@@ -6,6 +6,7 @@ use chrono::prelude::*;
 use chrono::Utc;
 
 use eyre::Result;
+use itertools::Itertools;
 
 use sqlx::sqlite::{
     SqliteConnectOptions, SqliteJournalMode, SqlitePool, SqlitePoolOptions, SqliteRow,
@@ -13,6 +14,7 @@ use sqlx::sqlite::{
 use sqlx::Row;
 
 use super::history::History;
+use super::ordering;
 use super::settings::SearchMode;
 
 #[async_trait]
@@ -280,12 +282,14 @@ impl Database for Sqlite {
         search_mode: SearchMode,
         query: &str,
     ) -> Result<Vec<History>> {
+        let orig_query = query;
         let query = query.to_string().replace("*", "%"); // allow wildcard char
         let limit = limit.map_or("".to_owned(), |l| format!("limit {}", l));
 
         let query = match search_mode {
             SearchMode::Prefix => query,
             SearchMode::FullText => format!("%{}", query),
+            SearchMode::Fuzzy => query.split("").join("%"),
         };
 
         let res = sqlx::query(
@@ -306,7 +310,7 @@ impl Database for Sqlite {
         .fetch_all(&self.pool)
         .await?;
 
-        Ok(res)
+        Ok(ordering::reorder_fuzzy(search_mode, orig_query, res))
     }
 
     async fn query_history(&self, query: &str) -> Result<Vec<History>> {
@@ -318,3 +322,109 @@ impl Database for Sqlite {
         Ok(res)
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    async fn new_history_item(db: &mut impl Database, cmd: &str) -> Result<()> {
+        let history = History::new(
+            chrono::Utc::now(),
+            cmd.to_string(),
+            "/home/ellie".to_string(),
+            0,
+            1,
+            Some("beep boop".to_string()),
+            Some("booop".to_string()),
+        );
+        return db.save(&history).await;
+    }
+
+    #[tokio::test(flavor = "multi_thread")]
+    async fn test_search_prefix() {
+        let mut db = Sqlite::new("sqlite::memory:").await.unwrap();
+        new_history_item(&mut db, "ls /home/ellie").await.unwrap();
+
+        let mut results = db.search(None, SearchMode::Prefix, "ls").await.unwrap();
+        assert_eq!(results.len(), 1);
+
+        results = db.search(None, SearchMode::Prefix, "/home").await.unwrap();
+        assert_eq!(results.len(), 0);
+
+        results = db.search(None, SearchMode::Prefix, "ls  ").await.unwrap();
+        assert_eq!(results.len(), 0);
+    }
+
+    #[tokio::test(flavor = "multi_thread")]
+    async fn test_search_fulltext() {
+        let mut db = Sqlite::new("sqlite::memory:").await.unwrap();
+        new_history_item(&mut db, "ls /home/ellie").await.unwrap();
+
+        let mut results = db.search(None, SearchMode::FullText, "ls").await.unwrap();
+        assert_eq!(results.len(), 1);
+
+        results = db
+            .search(None, SearchMode::FullText, "/home")
+            .await
+            .unwrap();
+        assert_eq!(results.len(), 1);
+
+        results = db.search(None, SearchMode::FullText, "ls  ").await.unwrap();
+        assert_eq!(results.len(), 0);
+    }
+
+    #[tokio::test(flavor = "multi_thread")]
+    async fn test_search_fuzzy() {
+        let mut db = Sqlite::new("sqlite::memory:").await.unwrap();
+        new_history_item(&mut db, "ls /home/ellie").await.unwrap();
+        new_history_item(&mut db, "ls /home/frank").await.unwrap();
+        new_history_item(&mut db, "cd /home/ellie").await.unwrap();
+        new_history_item(&mut db, "/home/ellie/.bin/rustup")
+            .await
+            .unwrap();
+
+        let mut results = db.search(None, SearchMode::Fuzzy, "ls /").await.unwrap();
+        assert_eq!(results.len(), 2);
+
+        results = db.search(None, SearchMode::Fuzzy, "l/h/").await.unwrap();
+        assert_eq!(results.len(), 2);
+
+        results = db.search(None, SearchMode::Fuzzy, "/h/e").await.unwrap();
+        assert_eq!(results.len(), 3);
+
+        results = db.search(None, SearchMode::Fuzzy, "/hmoe/").await.unwrap();
+        assert_eq!(results.len(), 0);
+
+        results = db
+            .search(None, SearchMode::Fuzzy, "ellie/home")
+            .await
+            .unwrap();
+        assert_eq!(results.len(), 0);
+
+        results = db.search(None, SearchMode::Fuzzy, "lsellie").await.unwrap();
+        assert_eq!(results.len(), 1);
+
+        results = db.search(None, SearchMode::Fuzzy, " ").await.unwrap();
+        assert_eq!(results.len(), 3);
+    }
+
+    #[tokio::test(flavor = "multi_thread")]
+    async fn test_search_reordered_fuzzy() {
+        let mut db = Sqlite::new("sqlite::memory:").await.unwrap();
+        // test ordering of results: we should choose the first, even though it happened longer ago.
+
+        new_history_item(&mut db, "curl").await.unwrap();
+        new_history_item(&mut db, "corburl").await.unwrap();
+        // if fuzzy reordering is on, it should come back in a more sensible order
+        let mut results = db.search(None, SearchMode::Fuzzy, "curl").await.unwrap();
+        assert_eq!(results.len(), 2);
+        let commands: Vec<&String> = results.iter().map(|a| &a.command).collect();
+        assert_eq!(commands, vec!["curl", "corburl"]);
+
+        results = db.search(None, SearchMode::Fuzzy, "xxxx").await.unwrap();
+        assert_eq!(results.len(), 0);
+
+        results = db.search(None, SearchMode::Fuzzy, "").await.unwrap();
+        assert_eq!(results.len(), 2);
+    }
+}
