@@ -1,6 +1,6 @@
 use std::path::PathBuf;
 
-use eyre::Result;
+use eyre::{Result, WrapErr};
 use structopt::StructOpt;
 
 use atuin_client::database::Sqlite;
@@ -96,8 +96,8 @@ pub enum AtuinCmd {
 
 impl AtuinCmd {
     pub async fn run(self) -> Result<()> {
-        let client_settings = ClientSettings::new()?;
-        let server_settings = ServerSettings::new()?;
+        let client_settings = ClientSettings::new().wrap_err("could not load client settings")?;
+        let server_settings = ServerSettings::new().wrap_err("could not load server settings")?;
 
         let db_path = PathBuf::from(client_settings.db_path.as_str());
 
@@ -108,7 +108,10 @@ impl AtuinCmd {
             Self::Import(import) => import.run(&mut db).await,
             Self::Server(server) => server.run(&server_settings).await,
             Self::Stats(stats) => stats.run(&mut db, &client_settings).await,
-            Self::Init(init) => init.run(),
+            Self::Init(init) => {
+                init.run();
+                Ok(())
+            }
             Self::Search {
                 cwd,
                 exit,
@@ -144,15 +147,14 @@ impl AtuinCmd {
                 logout::run();
                 Ok(())
             }
-            Self::Register(r) => register::run(
-                &client_settings,
-                r.username.as_str(),
-                r.email.as_str(),
-                r.password.as_str(),
-            ),
+            Self::Register(r) => {
+                register::run(&client_settings, &r.username, &r.email, &r.password)
+            }
             Self::Key => {
-                let key = atuin_client::encryption::load_key(&client_settings)?;
-                println!("{}", atuin_client::encryption::encode_key(key)?);
+                use atuin_client::encryption::{encode_key, load_key};
+                let key = load_key(&client_settings).wrap_err("could not load encryption key")?;
+                let encode = encode_key(key).wrap_err("could not encode encryption key")?;
+                println!("{}", encode);
                 Ok(())
             }
 
