@@ -11,17 +11,28 @@ use atuin_client::settings::Settings;
 #[structopt(setting(structopt::clap::AppSettings::DeriveDisplayOrder))]
 pub struct Cmd {
     #[structopt(long, short)]
-    pub username: String,
+    pub username: Option<String>,
 
     #[structopt(long, short)]
-    pub email: String,
+    pub email: Option<String>,
 
     #[structopt(long, short)]
-    pub password: String,
+    pub password: Option<String>,
 }
 
-pub fn run(settings: &Settings, username: &str, email: &str, password: &str) -> Result<()> {
-    let session = api_client::register(settings.sync_address.as_str(), username, email, password)?;
+pub fn run(
+    settings: &Settings,
+    username: &Option<String>,
+    email: &Option<String>,
+    password: &Option<String>,
+) -> Result<()> {
+    use super::login::or_user_input;
+    let username = or_user_input(username, "username");
+    let email = or_user_input(email, "email");
+    let password = or_user_input(password, "password");
+
+    let session =
+        api_client::register(settings.sync_address.as_str(), &username, &email, &password)?;
 
     let path = settings.session_path.as_str();
     let mut file = File::create(path)?;
